<?php
/*
Plugin Name: vimeorss
Version: 1.2
Plugin URI: http://ben.momillett.org/vimeorss/
Description: Creates a badge of recent videos from a vimeo user's account, now with image caching.
Author: Ben Millett
Author URI: http://benmillett.us/

Based on the del.icio.us plugin by Tom Gilbert (http://linuxbrit.co.uk)

Copyright (c) 2007
Released under the GPL license
http://www.gnu.org/licenses/gpl.txt
*/

/*
 version history:
 	v1.2 (2009-03) - Addressed another caching issue, hopefully making the temporary cache location universal
 	v1.1 (2009-03) - Addressed a caching issue
 	v1.0 (2009-01) - Added caching functionality
 	v0.4.1 (2008-11) - Fixed the thumbnail alt quotes so apostrophes in video titles don't invalidate xhtml.
 	v0.4 (2008-07) - Fixed a caching issue.  Previously, the collections would cache to the root folder.  Now they cache to the tmp folder (unless cache location manually set).
 	v0.3 () - added support for groups, channels, and albums
	v0.2 () - based on XML format
	v0.1 () - based on RSS format

*/

/*
 usage: 
 	Images are cached to the vimeorss plugins folder.  Create said folder if necessary.
 	If you don't want caching, set $useVImageCache below to 0.
*/

$useVImageCache = 1;  // SET TO 0 TO TURN OFF CACHING

/* 	<?php if (function_exists("vimeorss")) { vimeorss(); } ?>
 	with these options:
 	vimeorss(User name, Feed type (clips, subscriptions, contacts_like, contacts_clips, appears_in), Number of thumbnails, "Before image", "After image", Cache time length in seconds )
 	Style the output using CSS. Don't forget to enable
        the plugin on the wordpress admin page.


	There is no need to edit below this.
 */


function vimeorss(
  $vimeoxmluser,            	# your vimeoxml username
  $vimeoxmlrequest="clips",  #clips,subscriptions,contacts_like,contacts_clips,appears_in   
  $vcount=4,
  $vimeoxbefore="<li>",
  $vimeoxafter="</li>",
  $vcache_time=600,        	# how long to cache the results for (default=10 minutes)
  $cache_vfile = ""         #	cache file (defaults to /vimeopluginlocation/vimeoxml.$vimeoxmluser.$vcount.$request.$vcache_time.cache)
) {
  
	global $_fpvimeowrite, $_vimeoxml_user, $_curl_error_code;

	global $insideitemv, $vimeotag, $vimeotitle, $videouploader, $vimeourl, $vimeothumbnail, $vimeoxmlbefore, $vimeoxmlafter, $vxcount;
	
	global $itemsv;

	$insideitemv = FALSE;
	$vimeoxmlbefore = $vimeoxbefore;
	$vimeoxmlafter = $vimeoxafter;
	$vimeotag = "";
	$vimeotitle = "";
	$videouploader = "";
	$vimeourl = "";
	$itemsv = 0;
	$vxcount = $vcount;

	$_vimeoxml_user = $vimeoxmluser; 
  	$vapi_url = "http://vimeo.com/api/$vimeoxmluser/$vimeoxmlrequest.xml";
	
	if ($cache_vfile == "") 
	{
		$_vimeoxml_user = ereg_replace('/','-',$vimeoxmluser);
		
		if ( !function_exists('sys_get_temp_dir')) {
			function sys_get_temp_dir() {
				if (!empty($_ENV['TMP'])) { return realpath($_ENV['TMP']); }
				if (!empty($_ENV['TMPDIR'])) { return realpath( $_ENV['TMPDIR']); }
				if (!empty($_ENV['TEMP'])) { return realpath( $_ENV['TEMP']); }
				$tempfile=tempnam(uniqid(rand(),TRUE),'');
				if (file_exists($tempfile)) {
					unlink($tempfile);
					return realpath(dirname($tempfile));
				}
			}
		}
		if ($useVImageCache == 1) { $dirvimeocache = dirname(__FILE__); } 
		else { $dirvimeocache = sys_get_temp_dir(); }
		$cache_vfile  =  "$dirvimeocache/vimeoxml.$_vimeoxml_user.$vcache_time.cache";
		//$cache_vfile  =  "/tmp/vimeoxml.$vcache_time.cache";
	}
  	
	$cache_vfile_tmp = "$cache_vfile.tmp";

	$time = split(" ", microtime());
	srand((double)microtime()*1000000);
	
	# randomise a bit, between 30 and 60s "off the mark" to avoid a bunch of
	# requests all simultaneously deciding to refresh the file
	$vcache_time_rnd = 30 - rand(0, 60);

	if (
		!file_exists($cache_vfile)
	  	|| !filesize($cache_vfile) > 20
	  	|| ((filemtime($cache_vfile) + $vcache_time - $time[1]) + $vcache_time_rnd < 0)
	  	|| (filemtime(__FILE__) > filemtime($cache_vfile))
	  ) 
	{
	  
		$c = curl_init($vapi_url);
	  	curl_setopt($c, CURLOPT_RETURNTRANSFER,1); 
	  	//curl_setopt($c, CURLOPT_USERPWD,"$vimeoxmluser:$pass"); 
	  	curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 2);
	  	curl_setopt($c, CURLOPT_TIMEOUT, 4);
	  	curl_setopt($c, CURLOPT_USERAGENT, "vimeoxml WordPress Plugin");
	  	$response = curl_exec($c);
	  	$info = curl_getinfo($c);

		// check if the response contains actual clips
		$findme  = '<clip';
		$pos = strpos($response, $findme); 

		if ($pos !== false) 
		{
		
			//echo "write new cache file";
			
		  	$_curl_error_code = $info['http_code'];
		  	curl_close($c);
			if ($_curl_error_code == 200) // STATUS OK
			{
				$_fpvimeowrite = fopen($cache_vfile_tmp, 'w');
			 	if ($_fpvimeowrite) 
				{
					# parse the XML, then write out includable html to the cache file.
			   		if (!($vimeoxml_parser = xml_parser_create()))
			     		die("Couldn't create parser.");

			   		xml_set_element_handler($vimeoxml_parser,
			                           "startvElement",
			                           "endvElement");
			   		fputs($_fpvimeowrite, "\n");
					xml_set_character_data_handler($vimeoxml_parser, "charactervData"); 
			 		xml_parse($vimeoxml_parser, $response);
				 		fputs($_fpvimeowrite, "\n");

			   		xml_parser_free($vimeoxml_parser);
			   		fclose($_fpvimeowrite);
			   		# be atomic
					rename($cache_vfile_tmp, $cache_vfile);
				}
			}
		}
	}
		
	if ((file_exists($cache_vfile)) && filesize($cache_vfile) > 20) 
	{
		include($cache_vfile); //prints file
	} 
	elseif ($_curl_error_code) 
	{
		echo "<li>No recent clips (error $_curl_error_code)</li>";
	} 
	else 
	{
		echo "<li>No recent clips</li>";
	}
}

function startvElement($parser, $namev, $attrs) {
	global $insideitemv, $vimeotag, $vimeotitle, $videouploader, $vimeourl, $itemsv,$vxcount, $_fpvimeowrite;
	if ($insideitemv && $itemsv < $vxcount+1) 
	{
		$vimeotag = $namev;
	} 
	elseif ($namev == "CLIP") 
	{
		 $insideitemv = true;
		 $itemsv++;
		 
	}
}

function endvElement($parser, $namev) {
	global $insideitemv, $vimeotag, $vimeotitle, $videouploader, $vxcount, $itemsv, $vimeothumbnail, $vimeourl, $vimeoxmlbefore, $vimeoxmlafter, $_fpvimeowrite,$useVImageCache;
	if ($namev == "CLIP" && $itemsv < $vxcount+1) 
	{
	
	$fullvPath = ABSPATH.'wp-content/plugins/vimeorss/'; 
	$cachevPath = get_bloginfo('wpurl')."/wp-content/plugins/vimeorss/";
	preg_match('@^(?:http://)?([^(?j)]+)@i', $vimeothumbnail, $vSlugMatches);
	$vSlug = $vSlugMatches[1];
	$vSlug = ereg_replace('/','-',$vSlug);
	$vSlug = ereg_replace('\.','',$vSlug);


	if ($useVImageCache==1) {

		if (!file_exists("$fullvPath$vSlug.jpg")) {   
			if ( function_exists('curl_init') ) { 
				$cv = curl_init();
                $localvimage = fopen("$fullvPath$vSlug.jpg", "wb");
                curl_setopt($cv, CURLOPT_URL, $vimeothumbnail);
                curl_setopt($cv, CURLOPT_CONNECTTIMEOUT, 1);
                curl_setopt($cv, CURLOPT_FILE, $localvimage);
                curl_exec($cv);
                curl_close($cv);
            } else {
            	$filevdata = "";
                $remotevimage = fopen($vimeothumbnail, 'rb');
                if ($remotevimage) {
                	while(!feof($remotevimage)) {
                    	$filevdata.= fread($remotevimage,1024*8);
                    }
                }
            fclose($remotevimage);
            $localvimage = fopen("$fullvPath$vSlug.jpg", 'wb');
            fwrite($localvimage,$filevdata);
            fclose($localvimage);
            } 
        } 
    	$vimeothumbnail = "$cachevPath$vSlug.jpg";
 	} else {

		$vimeothumbnail = $vimeothumbnail;      
    }
		fputs($_fpvimeowrite, ''.$vimeoxmlbefore.'<a href="'.trim($vimeourl).'"><img alt="'.htmlspecialchars(trim($vimeotitle)).'" src="'.trim($vimeothumbnail).'"/></a>'.$vimeoxmlafter.'');
		$vimeotitle = "";
		$videouploader = "";
		$vimeourl = "";
		$vimeothumbnail = "";
		$insideitemv = false;
	}
}

function charactervData($parser, $data) {
	global $insideitemv, $vimeotag, $vimeotitle, $vimeothumbnail, $videouploader, $vimeourl;
	if ($insideitemv) 
	{
		switch ($vimeotag) {
			case "USER_NAME":
			$videouploader .= $data;
			break;
			case "THUMBNAIL_SMALL":
			$vimeothumbnail .= $data;
			break;
			case "TITLE":
			$vimeotitle .= $data;
			break;
			case "URL":
			$vimeourl .= $data;
			break;
		}
	}
}

?>